define([
        "dojo",
        "dijit",
        "dojo/window",
        "dojo/dom-geometry",
        "support/panels/Panel",
        "dojo/text!./templates/Dialog.html",
        "dijit/Dialog"
    ], function(dojo, dijit, modWindow, modDomGeom, panel, template) {

    return dojo.declare("support.dialogs.Dialog", [ support.panels.Panel ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // templateMode: [protected] String
        // The templateMode would be either new, edit, or copy
        // refers to the mode that the template is initially opened in
        templateMode: "new",

        // templateinUse: [protected] String
        // Template In use would be either true or false.
        // False meaning this template is not associated with any deployed profiles.
        // True meaning that the template is associated with 1 or more deployed profiles.
        templateInUse: false,

        // templateLaunchPoint: [protected] String
        // templateLaunchPoint refers to the place the template dialog was launched from
        // Use "wizard" if template dialog launched from Server Template Wizard
        // Use "table" if template dialog launched from Templates List (template.html)
        templateLaunchPoint: "",

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "supportDialog",

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/Dialog.css",

        // _setTitleAttr: [protected]
        // Pass "title" value along to dialogNode for display on dialog.
        _setTitleAttr: {
            node: "dialogNode"
        },

        // _setContentAttr: [protected]
        // Pass "content" value along to dialogContainerNode for display in dialog.
        _setContentAttr: {
            node: "dialogContainerNode"
        },

        // _subscriptions: [protected]
        // Container to store subscriptions placed on hold while the dialog is hidden.
        _subscriptions: null,

        // _messageStackNode: [protected]
        // Message area node for Dialog dijits
        _messageStackNode: null,

        constructor: function() {
            this._subscriptions = new Array();
        },
        
        postCreate: function() {
            this.inherited(arguments);
            this._resizeDialog();
        },

        _resizeDialog: function() {
            // IE gives spurious resize events and can actually get stuck
            // in an infinite loop if we don't ignore them
            // AA120928: So save the current viewport size, and if we're "
            // resizing" to the same size, ignore it.

            // If the dialog is hidden, bail.
            if (!this.dialogNode || !this.dialogNode.domNode)
                return;

            var viewportSize = modWindow.getBox();
            if (!this._oldViewportSize || viewportSize.h != this._oldViewportSize.h || viewportSize.w != this._oldViewportSize.w) {
                var contentOrWizardNode = this.contentNode ? this.contentNode : this._wizardNode;
                if (contentOrWizardNode) {
                    // AA120928-- A better way to do this would be to do a
                    // getContentBox() directly on dojo.Dialog's title node.
                    var fixedHeight = modDomGeom.getContentBox(this.dialogNode.domNode).h
                            - modDomGeom.getContentBox(contentOrWizardNode.domNode).h;
                    this._messageStackNode && (fixedHeight += modDomGeom.getContentBox(this._messageStackNode).h);

                    var height = (viewportSize.h - fixedHeight) * .90;
                    var width = viewportSize.w * .75;

                    // AA120928-- A better way to do this would be to set dojo.Dialog's
                    // doLayout, which would make the Dialog resize it's 1 and only child
                    // with an actual size, and then call Dialog's resize().
                    
                    // Resize the dialog contents based on the new viewport size.
                    contentOrWizardNode.resize({
                        w: width,
                        h: height
                    });

                    // Give the dialog a chance to resize it's contents.
                    this.resize();
                    // Center the dialog.
                    this.layout();
                }

                this._oldViewport = viewportSize;
            }
        },

        subscribe: function(/* String */topic, /* Function */method, /* boolean? */noRecord) {
            var handle = this.inherited(arguments);
            if (!noRecord) {
                // Record details for this subscription.
                this._subscriptions.push({
                    topic: topic,
                    method: method,
                    handle: handle
                });
                console.info(this.declaredClass + ": established subscription:", topic, handle);
            }
            return handle;
        },

        unsubscribe: function(/* Object */handle, /* boolean? */noRecord) {
            this.inherited(arguments);
            if (!noRecord) {
                // Discard record(s) for this handle.
                var indexes = new Array();
                dojo.forEach(this._subscriptions, function(item, idx) {
                    (item.handle === handle) && indexes.push(idx);
                });
                console.info(this.declaredClass + ": discarding subscription:", indexes);
                dojo.forEach(indexes, dojo.hitch(this, function(targetIdx, idx) {
                    this._subscriptions.splice(targetIdx, 1);
                }));
                console.info(this.declaredClass + ": remaining subscriptions:", this._subscriptions);
            }
        },

        show: function() {
            // summary:
            // show() is used to open the dialog node, displaying its contents.
            if (this.onShowDialog()) {
                this.dialogNode.show();

                // Go resize the dialog now to an appropriate size.
                this._resizeDialog();

                // When the viewport resizes, resize the dialog box.
                dojo.connect(window, "resize", this, this._resizeDialog);

                // Re-establish subscriptions placed "on hold" while the dialog was hidden.
                console.info(this.declaredClass + ": check subscriptions...");
                dojo.forEach(this._subscriptions, dojo.hitch(this, function(item, idx) {
                    if (!item.handle) {
                        console.info(this.declaredClass + ": re-establish subscription:", item.topic);
                        var handle = this.subscribe(item.topic, item.method, true);
                        item.handle = handle;
                    }
                    else {
                        // must have maintained subscription for some reason so do nothing.
                        console.info(this.declaredClass + ": maintained subscription:", item.topic);
                    }
                }));
            }
        },

        hide: function() {
            // summary:
            // hide() is used to hide the dialog node, along with all its contents.
            this.dialogNode.hide();

            // Place subscriptions "on hold" while the dialog is hidden.
            console.info(this.declaredClass + ": hold subscriptions...");
            dojo.forEach(this._subscriptions, dojo.hitch(this, function(item, idx) {
                // Unsubscribe to all topics while dialog is hidden, but keep record so
                // subscriptions may be re-established if the dialog is shown again.
                console.info(this.declaredClass + ": hold subscription:", item.topic);
                item.handle && this.unsubscribe(item.handle, true);
                item.handle = null;
            }));
        },

        destroy: function() {
            // summary:
            // destroy() is used to destroy the dialog node, along with all its contents, and
            // ourselves.
            this.dialogNode.destroyRecursive();
            this.inherited(arguments);
        },

        resize: function() {
            // summary:
            // resize() is used to resize our dialog node.
            this.dialogNode.resize();
            this.inherited(arguments);
        },

        layout: function() {
            // summary:
            // layout() is used to reposition our dialog node.
            if(this.CP_RUN_ENVIRONMENT == "fsm"){
                
                if(this.dialogNode)
                    this.dialogNode.layout();
                
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                if(this.dialogNode)
                    this.dialogNode.resize();
                
                if(this.contentNode)
                    this.contentNode.resize();                
            }
            this.inherited(arguments);
        },

        cssArrived: function() {
            // summary:
            // layout() is used to reposition our dialog node.
            this.resize();
            this.layout();
            this.inherited(arguments);
        },

        onFinish: function(evt) {
            // summary:
            // onFinished() is called when the dialog node is closed.
            // Does nothing by default. Override to specify custom handler.
            // NOTE: If desired (ie: due to error), return false to prevent dialog from closing.
            return true;
        },

        onShowDialog: function() {
            // summary:
            // onShowDialog() is called before the dialog node is opened.
            // Does nothing by default. Override to perform custom initialization. This is useful to
            // override when you want to initialize data or manipulate DOM nodes included with your
            // widget prior to the dialog opening.
            //
            // Many things typically done in postCreate() or postMixInProperties() phase may be done
            // by this method instead, effectively deferring this type of processing from creation
            // time to the time just before the dialog is shown.
            //
            // NOTE: If desired (ie: due to error), return false to prevent dialog from opening.
            return true;
        },

        _onFinish: function(evt) {
            // summary:
            // Do not override. Consider overriding onFinish() instead.
            // tags:
            // protected
            if (!this.onFinish(evt)) {
                return false;
            }
            return this._onClose();
        },

        _onClose: function(evt) {
            // summary:
            // Do not override. Consider overriding onClose() instead.
            // tags:
            // protected
            dojo.publish("CloseDialog");
            
            if (!this.onClose(evt)) {
                return false;
            }  
            if(this.dialogNode)
                this.hide();
            
            return true;
        },
        
        sortAndFlatten: function(obj) {
            var jsonOutput = dojo.toJson(this._sortObject(obj));
            
            console.info(this.declaredClass + ": sortAndFlatten:", jsonOutput);
            
            return jsonOutput;
        },
        
        _sortObject: function(obj) {
            var sortedArray = [];
            var objKeys = [];

            for (var key in obj) {
                (obj.hasOwnProperty(key) && (objKeys.push(key)));
            }
            objKeys.sort();
            for (var key in objKeys) {
                var val = obj[objKeys[key]];
                sortedArray.push((typeof val == "object") ? this._sortObject(val) : val);
                sortedArray.push(objKeys[key]);
            }
            return sortedArray;
        },
        
        postMessage: function(/*String*/type, /*String*/title, /*String*/description) {
            
            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require(["idx/oneui/messaging/SingleMessage"], dojo.hitch(this, function() {
                    if (!this._messageStackNode) {
                        // Place first, atop all else...
                        this._messageStackNode = dojo.place("<div style=\"max-width: 600px;\"></div>", this.dialogNode.containerNode, "first");
                    }
                    var msg = new idx.oneui.messaging.SingleMessage({
                        type: type,
                        title: title,
                        description: description,
                        dateFormat: {
                            datePattern: "dd MMMM y, ",
                            timePattern: "hh:mm a"
                        },
                        style: {
                            margin: "0px 0px 0px 0px"
                        },
                        showId: false,
                        showAction: false,
                        showRefresh: false,
                        showDetailsLink: false
                    });
                    dojo.connect(msg, "onClick", this, this._resizeDialog);
                    dojo.connect(msg, "onClose", this, this._resizeDialog);
                    msg.placeAt(this._messageStackNode); // Add to bottom of the message stack.
                    this._resizeDialog(); // Resize size a new message has been added.
                }));
            
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){
                
                require(["idx/widget/SingleMessage"], dojo.hitch(this, function() {
                    if (!this._messageStackNode) {
                        // Place first, atop all else...
                        this._messageStackNode = dojo.place("<div style=\"max-width: 600px;\"></div>", this.dialogNode.containerNode, "first");
                    }
                    var msg = new idx.widget.SingleMessage({
                        type: type,
                        title: title,
                        description: description,
                        dateFormat: {
                            datePattern: "dd MMMM y, ",
                            timePattern: "hh:mm a"
                        },
                        style: {
                            margin: "0px 0px 0px 0px"
                        },
                        showId: false,
                        showAction: false,
                        showRefresh: false,
                        showDetailsLink: false
                    });
                    dojo.connect(msg, "onClick", this, this._resizeDialog);
                    dojo.connect(msg, "onClose", this, this._resizeDialog);
                    msg.placeAt(this._messageStackNode); // Add to bottom of the message stack.
                    this._resizeDialog(); // Resize size a new message has been added.
                }));

            }
        }

    });
});